unit StringHelpers;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

const
  MaxSyllable = 100;

  StartSyllable: array[0..MaxSyllable - 1] of String = (
    'ver',
    'ge',
    'ei',
    'ab',
    'an',
    've',
    'be',
    'un',
    'aus',
    'auf',
    'ka',
    'ko',
    'au',
    'in',
    'vor',
    're',
    'wa',
    'he',
    'la',
    'na',
    'hi',
    'vo',
    'we',
    'zu',
    'ma',
    'da',
    'kon',
    'so',
    'um',
    'unter',
    'wi',
    'le',
    'mo',
    'fe',
    'en',
    'ta',
    'ar',
    'wei',
    'ha',
    'ba',
    'de',
    'al',
    'hoch',
    'bo',
    'fa',
    'to',
    'ki',
    'über',
    'ra',
    'zusam',
    'ga',
    'her',
    'pro',
    'po',
    'bu',
    'lan',
    'ku',
    'li',
    'unte',
    'wel',
    'no',
    'pe',
    'se',
    'ru',
    'ne',
    'bi',
    'grund',
    'haup',
    'einzel',
    'mu',
    'gesamt',
    'ange',
    'sta',
    'ausge',
    'spe',
    'auto',
    'frei',
    'kin',
    'per',
    'pla',
    'einge',
    'gru',
    'fah',
    'fu',
    'vers',
    'wir',
    'ober',
    'wet',
    'nie',
    'boden',
    'kla',
    'bil',
    'hinei',
    'abs',
    'wirt',
    'verb',
    'scha',
    'nord',
    'ste',
    'hau');

  MidSyllable: array[0..MaxSyllable - 1] of String = (
    'ge',
    'te',
    're',
    'be',
    'le',
    'de',
    'ra',
    'ti',
    'li',
    'ri',
    'se',
    'he',
    'ta',
    'na',
    'si',
    'we',
    'ste',
    'gen',
    'me',
    'ten',
    'ver',
    'ter',
    'rei',
    'di',
    'ne',
    'ren',
    'ni',
    'zu',
    'nde',
    'ma',
    'der',
    'ze',
    'ko',
    'nge',
    'ka',
    've',
    'sen',
    'men',
    'sse',
    'mi',
    'ha',
    'tu',
    'pi',
    'fe',
    'bi',
    'che',
    'sta',
    'to',
    'ke',
    'sti',
    'sie',
    'len',
    'pe',
    'rge',
    'schi',
    'ro',
    'nen',
    'ga',
    'wa',
    'lo',
    'ba',
    'nte',
    'ber',
    'tte',
    'tie',
    'tions',
    'no',
    'lle',
    'po',
    'den',
    'for',
    'rte',
    'chi',
    'lie',
    'zie',
    'nie',
    'bel',
    'ges',
    'reic',
    'her',
    'ger',
    'nter',
    'hen',
    'bo',
    'mme',
    'nder',
    'hal',
    'rin',
    'so',
    'lu',
    'cke',
    'bei',
    'ssen',
    'llen',
    'ran',
    'nzu',
    'kon',
    'des',
    'tur',
    'tell');

  EndSyllable: array[0..MaxSyllable - 1] of String = (
    'ten',
    'gen',
    'te',
    'nden',
    'ndes',
    'ngen',
    'nen',
    'nder',
    'den',
    'des',
    'ndem',
    'de',
    'ren',
    'der',
    'ne',
    'tes',
    're',
    'rten',
    'le',
    'ge',
    'rte',
    'len',
    'ter',
    'rung',
    'chen',
    'hen',
    'llen',
    'lten',
    'sen',
    'sten',
    'men',
    'tion',
    'chten',
    'ges',
    'che',
    'se',
    'rin',
    'tem',
    'tionen',
    'schen',
    'lte',
    'ste',
    'ches',
    'rtes',
    'test',
    'tet',
    'gungen',
    'scher',
    'ken',
    'rem',
    'rtem',
    'ger',
    'me',
    'hes',
    'stes',
    'ses',
    'cken',
    'zen',
    'tung',
    'lung',
    'lter',
    'ment',
    'ltes',
    'gem',
    'mmen',
    'les',
    'tern',
    'heit',
    'ters',
    'tur',
    'stand',
    'her',
    'ke',
    'turen',
    'sens',
    'teils',
    'nlagen',
    'tor',
    'rens',
    'rnden',
    'stem',
    'bes',
    'fe',
    'dung',
    'gkeiten',
    'satz',
    'nung',
    'hem',
    'chtung',
    'hens',
    'bieten',
    'wegung',
    'ssung',
    'trags',
    'bung',
    'htes',
    'nste',
    'tik',
    'räten',
    'bens');

function RandomString: String;
function RandomStringSimple(MaxLength: Integer = 0): String;
function RandomSentence: String;

implementation

function RandomString: String;
var
  i: Integer;
begin
  Result := StartSyllable[Random(MaxSyllable)];
  if Random(2) > 0 then
  for i := 0 to Random(4) do
    Result := Result + MidSyllable[Random(MaxSyllable)];
  if Random(3) > 0 then
    Result := Result + EndSyllable[Random(MaxSyllable)];
end;

function RandomStringSimple(MaxLength: Integer): String;
const
  Charset = 'abcdefghijklmnopqrstuvwxyz';
var
  i, Cnt: Integer;
begin
  if MaxLength > 0 then
    Cnt := Random(MaxLength) + 1
  else
    Cnt := Random(20) + 2;
  SetLength(Result, Cnt);
  for i := 1 to Length(Result) do
    Result[i] := Charset[Random(Length(Charset)) + 1];
end;

function RandomSentence: String;
var
  i: Integer;
begin
  Result := RandomString;
  for i := 0 to Random(20) + 2 do
    Result := Result + ' ' + RandomString;
  Result := Uppercase(Copy(Result, 1, 1)) + Copy(Result, 2, Length(Result)) + '.';
end;

end.

