unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, 
    Dialogs, VirtualTrees, DelphiCompat, StringHelpers, Types;

type

  PTreeData = ^TTreeData;
  TTreeData = record
    Str0: String;
    Str1: String;
    Str2: String;
    Str3: String;
    Str4: String;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Tree: TVirtualStringTree;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TreeFreeNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure TreeGetNodeDataSize(Sender: TBaseVirtualTree;
      var NodeDataSize: Integer);
    procedure TreeGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex;
      {%H-}TextType: TVSTTextType; var CellText: String);
    procedure TreeNewText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex;
      const NewText: String);
  private
    procedure InitTreeWidths;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
  Data: PTreeData;
  Node: PVirtualNode;
begin
  Tree.BeginUpdate;
  for i := 0 to 99 do begin
    Node := Tree.AddChild(nil);
    Data := Tree.GetNodeData(Node);
    Data^.Str0 := RandomString;
    Data^.Str1 := IntToStr(Random(100));
    Data^.Str2 := RandomSentence;
    Data^.Str3 := RandomStringSimple(10);
    Data^.Str4 := RandomString;
  end;
  Tree.EndUpdate;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  InitTreeWidths;
end;

procedure TForm1.TreeFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PTreeData;
begin
  Data := Tree.GetNodeData(Node);
  if not Assigned(Data) then Exit;
  Data^.Str0 := '';
  Data^.Str1 := '';
  Data^.Str2 := '';
  Data^.Str3 := '';
  Data^.Str4 := '';
end;

procedure TForm1.TreeGetNodeDataSize(
  Sender: TBaseVirtualTree; var NodeDataSize: Integer);
begin
  NodeDataSize := SizeOf(TTreeData);
end;

procedure TForm1.TreeGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType; var CellText: String);
var
  Data: PTreeData;
begin
  Data := Tree.GetNodeData(Node);
  if not Assigned(Data) then Exit;
  case Column of
    0: CellText := Data^.Str0;
    1: CellText := Data^.Str1;
    2: CellText := Data^.Str2;
    3: CellText := Data^.Str3;
    4: CellText := Data^.Str4;
  end;
end;

procedure TForm1.TreeNewText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex;
  const NewText: String);
var
  Data: PTreeData;
begin
  Data := Tree.GetNodeData(Node);
  if not Assigned(Data) then Exit;
  case Column of
    0: Data^.Str0 := NewText;
    1: Data^.Str1 := NewText;
    2: Data^.Str2 := NewText;
    3: Data^.Str3 := NewText;
    4: Data^.Str4 := NewText;
  end;
end;

procedure TForm1.InitTreeWidths;
var
  c: UInt32;
  ws: String;
  cnv: TCanvas;
  Fnt: TFont;
  ThisWidth: UInt32;
  MinWidth: UInt32;
  Node: PVirtualNode;
  r: TRect;
  s: TSize;
begin
  cnv := Tree.Canvas;
  for c := 0 to Tree.Header.Columns.Count - 1 do begin
    ws := Tree.Header.Columns[c].Text;
    GetTextExtentPoint32W(cnv.Handle, PWideChar(ws), Length(ws), s);
    if c = 0 then
      MinWidth := s.cx + 30
    else
      MinWidth := s.cx + 10;
    Fnt := Tree.Font;
    Node := Tree.GetFirst;
    while Assigned(Node) do begin
      Tree.GetTextInfo(Node, c, Fnt, r{%H-}, ws);
      if c = 0 then
        ThisWidth := r.Right - r.Left + 30
      else
        ThisWidth := r.Right - r.Left + 10;
      if MinWidth < ThisWidth then MinWidth := ThisWidth;
      Node := Tree.GetNext(Node);
    end;
    Tree.Header.Columns[c].Width := MinWidth;
  end;
end;

end.

